/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef CONFIGMANAGER_H__
#define CONFIGMANAGER_H__

#include "macros.h"

#include <gloox/gloox.h>
#include <gloox/mutex.h>

#include <list>

namespace gloox
{
  class JID;
  class LogSink;
}

namespace glooxd
{

  class ConfigHandler;

  /**
   * @brief The manager for the server configuration.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API ConfigManager
  {
    public:
      /**
       * Constructor.
       * @param _logInstance The LogSink to use.
       */
      ConfigManager( const gloox::LogSink& _logInstance );

      /**
       * Virtual destructor.
       */
      virtual ~ConfigManager();

      /**
       * Registers a new ConfigHandler.
       * @param ch A new ConfigHandler.
       */
      void registerConfigHandler( ConfigHandler* ch );

      /**
       * Use this function to add a domain to a previously added
       * interface. That is, use addC2SInterface() first, the
       * bindDomain() (using the same interface/port).
       * @param domain A domain to handle.
       * @param interface A local interface to bind to. Default: all
       * local interfaces.
       * @param port A local port to listen on. Default: 5222
       */
      void bindDomain( const std::string& domain, const std::string& interface = gloox::EmptyString,
                      unsigned short port = 5222 );

      /**
       * Use this function to remove a domain from the given interface.
       * @param domain A domain to remove.
       * @param interface A local interface.
       * @param port A local port.
       */
      void unbindDomain( const std::string& domain, const std::string& interface = gloox::EmptyString,
                       unsigned short port = 5222 );

      /**
       * Adds a C2S interface that can then be used with bindDomain().
       * @param interface The interface to bind to.
       * @param port The port to listen on.
       */
      bool addC2SInterface( const std::string& interface = gloox::EmptyString,
                            unsigned short port = 5222 );

      /**
       * Stops listening on the given interface/port.
       * @param interface The interface.
       * @param port The port.
       */
      bool removeC2SInterface( const std::string& interface = gloox::EmptyString,
                               unsigned short port = 5222 );

      /**
       * Causes the server to rebind its local interfaces. This is
       * useful if one or more listening interfaces are 'wildcards',
       * ie. the server is listening on all local interfaces, and an
       * interface has been added to or removed from the system.
       */
      void rebind();

      /**
       * Prepares a system shutdown, that is, stops all listening
       * interfaces and disconnects any connected clients,
       * indicating that the disconnect is due to a server shutdown.
       */
      void shutdown();

      /**
       * Checks if the given domain is valid for the given interface/port.
       * @param domain The domain to check.
       * @param interface The local interface.
       * @param port The local port.
       */
      virtual bool checkDomain( const std::string& domain, const std::string& ip,
                                int port );

    private:
      /**
       * An interface describing a network interface to listen on.
       */
      struct Interface
      {

        public:
          /**
           * Creates a new interface description.
           * @param iface The numeric IPv4 address of a local
           * interface, defaults
           * to all interfaces.
           * @param p he port to listen on, defaults to 5222.
           */
          Interface( const std::string& iface = gloox::EmptyString, int p = 5222 )
            : m_interface( iface ), m_port( p )
          {}

          /**
           * Destructor.
           */
          ~Interface() {}

          /**
           * Returns the current interface string.
           * @return The current interface string.
           */
          const std::string& interface() const { return m_interface; }

          /**
           * Returns the current port.
           * @return The current port.
           */
          int port() const { return m_port; }

          /**
           * Checks whether the interface handles the given domain.
           * @return @b True if the interface handles the domain,
           * @b false otherwise.
           */
          bool hasDomain( const std::string& domain );

          /**
           * Removes the given domain from the interface.
           * @param domain The domain to remove.
           */
          void removeDomain( const std::string& domain )
          {
            m_domains.remove( domain );
          }

          /**
           * Adds the given domain from the interface.
           * @param domain The domain to add.
           */
          void addDomain( const std::string& domain )
          {
            m_domains.push_back( domain );
          }

        private:
          std::string m_interface;
          int m_port;

          gloox::StringList m_domains;

      };

      bool findInterface( const std::string& interface, unsigned short port );

      typedef std::list<ConfigHandler*> ConfigHandlerList;
      typedef std::list<Interface*> InterfaceList;

      ConfigHandlerList m_configHandlers;
      InterfaceList m_c2sInterfaces;

      gloox::util::Mutex m_c2sInterfacesMutex;
      gloox::util::Mutex m_configHandlersMutex;

      const gloox::LogSink& m_logInstance;

  };

}

#endif // CONFIGMANAGER_H__
